@@ -28,12 +28,13 @@
 #' @export
 #' @examples \dontrun{
 #' spp <- c('Danaus plexippus','Accipiter striatus','Pinus contorta')
-#' dat <- occ(spp, from='gbif', gbifopts=list(hasCoordinate=TRUE))
-#' dat <- fixnames(dat)
-#' df <- occ2df(dat)
+#' dat <- occ(spp, from=c('gbif','ecoengine'), gbifopts=list(hasCoordinate=TRUE))
+#' dat <- fixnames(dat, "query")
 #' 
 #' # Define colors
-#' mapgist(data=df, color=c('#976AAE','#6B944D','#BD5945'))
+#' mapgist(data=dat, color=c('#976AAE','#6B944D','#BD5945'))
+#' mapgist(data=dat$gbif, color=c('#976AAE','#6B944D','#BD5945'))
+#' mapgist(data=dat$ecoengine, color=c('#976AAE','#6B944D','#BD5945'))
 #' 
 #' # Define colors and marker size
 #' mapgist(data=df, color=c('#976AAE','#6B944D','#BD5945'), size=c('small','medium','large'))
@@ -43,7 +44,10 @@
 #' }
 
 mapgist <- function(data, description = "", file = "gistmap", dir = NULL, browse = TRUE, 
-                    ...) {
+                    ...) 
+{
+  assert_that(is(data, "occdatind")|is(data, "occdat"))
+  data <- if(is(data, "occdatind")) do.call(rbind, data$data) else occ2df(data)
   if (is.null(dir)) 
     dir <- paste0(getwd(), "/")
   spplist <- as.character(unique(data$name))
